library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity Epm3064ChristmasLights is
	port (
		Clk  : in std_logic;
		Led1 : out std_logic;
		Led2 : out std_logic;
		Led3 : out std_logic;
		Led4 : out std_logic;
		Led5 : out std_logic
	);
end entity;

architecture A of Epm3064ChristmasLights is
	constant COUNTER_LOAD : integer := 50000000;   -- 1 second at 50 MHz
	signal LFSRDBus : std_logic_vector(9 downto 0);
	signal LFSRQBus : std_logic_vector(9 downto 0);
	signal CounterDBus : std_logic_vector(25 downto 0);
	signal CounterQBus : std_logic_vector(25 downto 0);
	signal CounterIsZero : std_logic;
begin
    -- LFSR
	process (Clk)
	begin 
		if (Clk'event and (Clk = '1')) then
			LFSRQBus <= LFSRDBus;
		end if;
	end process;
	
	LFSRDBus <= "0000000001" when (to_integer(unsigned(LFSRQBus)) = 0) else
	            ((LFSRQBus(3) xor LFSRQBus(0)) & LFSRQBus(9 downto 1)) when (CounterIsZero = '1') else
				LFSRQBus;

	-- Counter
	process (Clk)
	begin
		if (Clk'event and (Clk = '1')) then
			CounterQBus <= CounterDBus;
		end if;
	end process;
	
	CounterDBus <= std_logic_vector(to_unsigned(COUNTER_LOAD, 26)) when (CounterIsZero = '1') else
	               std_logic_vector(to_unsigned(to_integer(unsigned(CounterQBus)) - 1, 26));

	CounterIsZero <= '1' when (to_integer(unsigned(CounterQBus)) = 0) else
	                 '0';
	Led1 <= LFSRQBus(9);
	Led2 <= LFSRQBus(7);
	Led3 <= LFSRQBus(5);
	Led4 <= LFSRQBus(3);
	Led5 <= LFSRQBus(1);
end architecture;
